@@ -1028,7 +1028,6 @@ paint_all (Display *dpy, XserverRegion region)
 	{
 	    w->borderClip = XFixesCreateRegion (dpy, NULL, 0);
 	    XFixesCopyRegion (dpy, w->borderClip, region);
-	    XFixesIntersectRegion(dpy, w->borderClip, w->borderClip, w->borderSize);
 	}
 	w->prev_trans = t;
 	t = w;
@@ -1080,6 +1079,8 @@ paint_all (Display *dpy, XserverRegion region)
 	if (w->mode == WINDOW_TRANS)
 	{
 	    int	x, y, wid, hei;
+	    XFixesIntersectRegion(dpy, w->borderClip, w->borderClip, w->borderSize);
+	    XFixesSetPictureClipRegion(dpy, rootBuffer, 0, 0, w->borderClip);
 #if HAS_NAME_WINDOW_PIXMAP
 	    x = w->a.x;
 	    y = w->a.y;
@@ -1099,6 +1100,8 @@ paint_all (Display *dpy, XserverRegion region)
 	else if (w->mode == WINDOW_ARGB)
 	{
 	    int	x, y, wid, hei;
+	    XFixesIntersectRegion(dpy, w->borderClip, w->borderClip, w->borderSize);
+	    XFixesSetPictureClipRegion(dpy, rootBuffer, 0, 0, w->borderClip);
 #if HAS_NAME_WINDOW_PIXMAP
 	    x = w->a.x;
 	    y = w->a.y;
@@ -2337,8 +2340,17 @@ main (int argc, char **argv)
 		    if (w)
 		    {
 			if (fadeTrans)
-			    set_fade (dpy, w, w->opacity*1.0/OPAQUE, get_opacity_percent (dpy, w, 1.0),
-				      fade_out_step, NULL, False, True, False);
+			{
+				double start, finish, step;
+				start = w->opacity*1.0/OPAQUE;
+				finish = get_opacity_percent (dpy, w, 1.0);
+				if(start > finish)
+					step = fade_in_step;
+				else
+					step = fade_out_step;
+				set_fade (dpy, w, start, finish, step,
+						NULL, False, True, False);
+			}
 			else
 			{
 			w->opacity = get_opacity_prop(dpy, w, OPAQUE);
